use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007006
use Test::Spelling 0.17;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
1nickt
AEST
Alders
Anno
Anwar
BCE
Bell
Ben
Bennett
BooK
Book
Bowen
Bruhat
CLDR
CPAN
Ceccarelli
Christian
Conflicts
Conrad
DATETIME
DROLSKY
DROLSKY's
Daisuke
Dan
DateTime
DateTimeX
DateTimes
Dave
David
Davis
Domini
Doug
Duration
Dyck
EEEE
EEEEE
Etheridge
Flávio
Formatters
GGGG
GGGGG
Gianni
Glock
Gregory
Hansen
Hant
Hauke
Helpers
Hill
Hoblitt
Howarth
IEEE
Iain
Infinite
James
Jason
Joshua
Karen
Kington
LLL
LLLL
LLLLL
LeapSecond
Liang
Liang's
MMM
MMMM
MMMMM
Maki
Mark
McIntosh
Measham
Measham's
MetaCPAN
Michael
Mohammad
Nick
Olaf
Oschwald
Overmeer
Ovid
POSIX
PP
PPExtra
Paul
PayPal
Philippe
Postgres
Precious
QQQ
QQQQ
Raspass
Rata
Ricardo
Richard
Rolsky
Rolsky's
Ron
SU
Sam
Signes
Soibelmann
Somerville
Stewart
Storable
TW
TZ
Tonkin
Truskett
Tsai
Types
UTC
VVVV
Wheeler
YAPCs
ZZZZ
ZZZZZ
afterwards
autarch
bian
book
bowen
brenan
ccc
cccc
ccccc
chansen
conformant
curtis_ovid_poe
danielandrewstewart
datetime
datetime's
datetimes
david
davidp
deceased
decrement
dian
dmaki
dracos
drolsky
durations
eee
eeee
eeeee
env
ether
fallback
fglock
fiji
formatter
gianni
github
grinnz
haukex
hh
iCal
jhoblitt
ji
jmac
jraspass
lib
madcityzen
mark
mike
mohammad
mrdvt92
mutiplication
na
namespace
ni
nitty
olaf
oschwald
other's
paul
philip
philiprbrenan
proleptic
qqq
qqqq
rjbs
rkhill
sexagesimal
subclasses
tidyable
uu
viviparous
vvvv
wiki
yy
yyyy
yyyyy
zh
zzzz
